import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import java.awt.Font;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.LinkedHashMap;
import javax.swing.JFrame;

public class Editor {
	private static EditorWindow editorWindow;
	private static final Font uiFont = new Font("Segoe UI", Font.PLAIN, 12);  // Move to preference manager
	private static List<XSlot> slots = new ArrayList<XSlot>();
	private static Map<String, XElement> elements = new LinkedHashMap<String, XElement>();
	private static Map<String, XSlotMapping> slot_mappings = new LinkedHashMap<String, XSlotMapping>();

	public static EditorWindow getEditorWindow() {
		return editorWindow;
	}

	public static void pushTreeUpdate() {
		editorWindow.updateTree();
	}

	public static void pushSelectedComponentUpdate() {
		editorWindow.updateSelectedComponent();
	}

	public static JFrame getEditorWindowFrame() {
		return (JFrame)editorWindow;
	}

	public static Font getUIFont() {
		return uiFont;
	}

	public static void addSlot(XSlot slot) {
		slots.add(slot);
	}

	public static List<XSlot> getSlots() {
		return slots;
	}

	public static void clearSlots() {
		slots.clear();
	}

	public static void addElement(String id, XElement element) {
		elements.put(id, element);
	}

	public static Map<String, XElement> getElements() {
		return elements;
	}

	public static void clearElements() {
		elements.clear();
	}

	public static void addSlotMapping(String id, XSlotMapping mapping) {
		slot_mappings.put(id, mapping);
	}

	public static Map<String, XSlotMapping> getSlotMappings() {
		return slot_mappings;
	}

	public static void clearSlotMappings() {
		slot_mappings.clear();
	}

	public static void main(String[] args) {
		PreferenceManager.loadPreferences();
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try { UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); } catch(Exception ex) {}
				editorWindow = new EditorWindow();
			}
		});
	}
}