import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.awt.event.*;
import java.awt.*;
import org.jdom2.*;
import org.jdom2.input.*;
import org.jdom2.output.*;
import javax.swing.plaf.basic.BasicInternalFrameUI;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;
import java.awt.datatransfer.DataFlavor;

class EditorWindow extends JFrame implements ActionListener {
	private JMenuBar menuBar;
	private JMenuItem menu_file_new, menu_file_open, menu_file_save, menu_file_saveas, menu_file_exit, menu_edit_slots, menu_elements_remove, menu_edit_mappings, menu_edit_prefs;
	private JSplitPane splitPane, splitPane2, splitPaneRight;
	private JTree tree;
	private DefaultMutableTreeNode tree_top;
	private JPanel split_right;
	private JDesktopPane desktop;
	private XElementBranch root;
	private PropertyTableModel propertyModel;
	private SignalTableModel signalModel;
	private XElement selectedElement = null;
	private String save_location = null;

	public void updateTree() {
		this.tree.updateUI();
		System.out.println("updateTree");
	}

	public void updateSelectedComponent() {
		this.selectedElement.updateComponent();
		System.out.println("updateSelectedComponent");
	}

	public Container getWindowVisualiserContentPane() {
		return ((WindowVisualiser)root.getComponent()).getContentPane();
	}

	public WindowVisualiser getWindowVisualiser() {
		return ((WindowVisualiser)root.getComponent());
	}

	public EditorWindow() {
		// Set up main window
		setTitle("xGUI Editor");
		setSize(1050, 750);
		setLocationRelativeTo(null);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		ImageIcon img = new ImageIcon("icon.png");
		setIconImage(img.getImage());

		// Set up menu bar
		menuBar = new JMenuBar();

		JMenu menu_file = new JMenu("File");
		menuBar.add(menu_file);

		menu_file_new = new JMenuItem("New", KeyEvent.VK_N);
		menu_file_new.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		menu_file_new.addActionListener(this);
		menu_file.add(menu_file_new);

		menu_file_open = new JMenuItem("Open...", KeyEvent.VK_O);
		menu_file_open.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		menu_file_open.addActionListener(this);
		menu_file.add(menu_file_open);

		menu_file_save = new JMenuItem("Save", KeyEvent.VK_S);
		menu_file_save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		menu_file_save.addActionListener(this);
		menu_file.add(menu_file_save);

		menu_file_saveas = new JMenuItem("Save As...", KeyEvent.VK_S);
		menu_file_saveas.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK | ActionEvent.CTRL_MASK));
		menu_file_saveas.addActionListener(this);
		menu_file.add(menu_file_saveas);

		menu_file_exit = new JMenuItem("Exit", KeyEvent.VK_E);
		menu_file_exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
		menu_file_exit.addActionListener(this);
		menu_file.add(menu_file_exit);

		JMenu menu_edit = new JMenu("Edit");
		menuBar.add(menu_edit);

		menu_edit_slots = new JMenuItem("Slots...");
		menu_edit_slots.addActionListener(this);
		menu_edit.add(menu_edit_slots);

		menu_edit_mappings = new JMenuItem("Mappings...");
		menu_edit_mappings.addActionListener(this);
		menu_edit.add(menu_edit_mappings);
		
		menu_edit_prefs = new JMenuItem("Preferences...");
		menu_edit_prefs.addActionListener(this);
		menu_edit.add(menu_edit_prefs);

		
		JMenu menu_elements = new JMenu("Elements");
		menuBar.add(menu_elements);

		menu_elements_remove = new JMenuItem("Remove Element");
		menu_elements_remove.setEnabled(false);
		menu_elements_remove.addActionListener(this);
		menu_elements.add(menu_elements_remove);

		setJMenuBar(menuBar);

		//Create element tree
		tree_top = new DefaultMutableTreeNode("GUI Elements");
		tree = new JTree(tree_top);
		tree.setBorder(new javax.swing.border.EmptyBorder(3, 3, 3, 3));
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setFont(Editor.getUIFont());
		tree.setLargeModel(true);
		tree.setRowHeight(19);

		tree.setCellRenderer(new ElementTreeIconRenderer());
		
		tree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();

				menu_elements_remove.setEnabled(node != null && node.getLevel() > 1);

				if (node == null) return;

				propertyModel.clear();
				signalModel.clear();
				selectedElement = null;
				if (node.getLevel() != 0) {
					selectedElement = (XElement)node.getUserObject();
					propertyModel.update(selectedElement.getProperties());
					signalModel.update(selectedElement.getProperty("id").getValue().toString(), selectedElement.getSignals());
				}
			}
		});

		JScrollPane treeView = new JScrollPane(tree);
		treeView.setBorder(BorderFactory.createEmptyBorder());

		//Set up split pane and panels
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setBorder(BorderFactory.createEmptyBorder());

		splitPane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

		splitPane2.setTopComponent(treeView);
		splitPane2.setBorder(BorderFactory.createEmptyBorder());

		// Set up property inspector
		propertyModel = new PropertyTableModel();
		
		JTable table = new JTable(propertyModel) {
			public TableCellEditor getCellEditor(int row, int column) {
				return propertyModel.getCellEditor(row);
			}
		};
		
		
		table.getTableHeader().setReorderingAllowed(false);
		table.setFont(Editor.getUIFont());
		table.getTableHeader().setFont(Editor.getUIFont());
		table.setGridColor(new Color(220, 220, 220));
		table.setRowHeight(20);

		JScrollPane scrollPaneProperties = new JScrollPane(table);
		scrollPaneProperties.setBorder(BorderFactory.createEmptyBorder());
		scrollPaneProperties.setBorder(BorderFactory.createEmptyBorder());

		table.setFillsViewportHeight(true);


		// Set up signal inspector
		signalModel = new SignalTableModel();

		JTable tableSignals = new JTable(signalModel) {
			public TableCellEditor getCellEditor(int row, int column) {
				JComboBox mappings = new JComboBox(new String[] { "None" });
				for (Object mapping : Editor.getSlotMappings().keySet().toArray())
					mappings.addItem(mapping);
				mappings.addItem("New Slot Mapping...");
				mappings.setFont(Editor.getUIFont());
				DefaultCellEditor dce1 = new DefaultCellEditor(mappings);
				return dce1;
			}
		};

		tableSignals.getTableHeader().setReorderingAllowed(false);
		tableSignals.setFont(Editor.getUIFont());
		tableSignals.getTableHeader().setFont(Editor.getUIFont());
		tableSignals.setGridColor(new Color(220, 220, 220));
		tableSignals.setRowHeight(20);
		tableSignals.setFillsViewportHeight(true);
		tableSignals.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);


		JScrollPane scrollPaneSignals = new JScrollPane(tableSignals);
		scrollPaneSignals.setBorder(BorderFactory.createEmptyBorder());


		// Set up element editing tabs
		JTabbedPane tabElement = new JTabbedPane();
		tabElement.setFont(Editor.getUIFont());
		tabElement.addTab("Properties", new ImageIcon("icons/properties.png"), scrollPaneProperties, null);
		tabElement.addTab("Signals", new ImageIcon("icons/signal.png"), scrollPaneSignals, null);


		splitPane.setLeftComponent(splitPane2);


		desktop = new JDesktopPane();
		desktop.setBackground(new Color(66, 94, 138));

		splitPaneRight = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPaneRight.setBorder(BorderFactory.createEmptyBorder());
		splitPaneRight.setLeftComponent(desktop);


		//Set up tree of available components
		DefaultMutableTreeNode component_tree_top = new DefaultMutableTreeNode("Components");
		component_tree_top.add(new DefaultMutableTreeNode("Button"));
		component_tree_top.add(new DefaultMutableTreeNode("Text Box"));
		component_tree_top.add(new DefaultMutableTreeNode("Image Box"));
		component_tree_top.add(new DefaultMutableTreeNode("Label"));
		component_tree_top.add(new DefaultMutableTreeNode("Panel"));
		component_tree_top.add(new DefaultMutableTreeNode("Group Box"));

		/*component_tree_top.add(new DefaultMutableTreeNode("Password Box"));
		component_tree_top.add(new DefaultMutableTreeNode("Progress Bar"));
		component_tree_top.add(new DefaultMutableTreeNode("Horizontal Splitter"));
		component_tree_top.add(new DefaultMutableTreeNode("Vertical Splitter"));
		component_tree_top.add(new DefaultMutableTreeNode("Checkbox"));
		component_tree_top.add(new DefaultMutableTreeNode("Radio Button"));
		component_tree_top.add(new DefaultMutableTreeNode("Dropdown"));*/

		JTree component_tree = new JTree(component_tree_top);
		component_tree.setBorder(new javax.swing.border.EmptyBorder(3, 3, 3, 3));
		component_tree.setDragEnabled(true);
		component_tree.setCellRenderer(new ComponentTreeIconRenderer());
		component_tree.setFont(Editor.getUIFont());
		component_tree.setRowHeight(19);

		JScrollPane componentTreeView = new JScrollPane(component_tree);
		componentTreeView.setBorder(BorderFactory.createEmptyBorder());
		splitPaneRight.setRightComponent(componentTreeView);

		splitPane.setRightComponent(splitPaneRight);

		//Set up split panes
		//splitPane2.setBottomComponent(scrollPaneProperties);
		splitPane2.setBottomComponent(tabElement);
		add(splitPane);

		splitPane.setDividerSize(5);
		splitPane2.setDividerSize(5);
		splitPaneRight.setDividerSize(5);
		splitPaneRight.setDividerSize(5);
		//splitPane.setDividerLocation(0.25);

		setVisible(true);
		reset();
		arrangeSplitPanels();
	}

	public void addElement(XElement element) {
		IDHandler.add(element.getProperty("id").getValue().toString());
		Editor.addElement(element.getProperty("id").getValue().toString(), element);
		root.addChild(element);

		((WindowVisualiser)root.getComponent()).add(element.getComponent());
		((WindowVisualiser)root.getComponent()).updateUI();

		DefaultMutableTreeNode treeNode = new DefaultMutableTreeNode(element);
		((DefaultMutableTreeNode)tree_top.getChildAt(0)).add(treeNode);
		tree.updateUI();
	}

	private void handleNode(Element elem, DefaultMutableTreeNode parentNode, XElementBranch parent) {
		String id = elem.getAttribute("id").getValue();
		IDHandler.add(id);

		XElement new_element = ElementFactory.create(elem);

		if (root == null)
			root = (XElementBranch)new_element;
		else
			parent.addChild(new_element);

		DefaultMutableTreeNode tree_window = new DefaultMutableTreeNode(new_element);
		parentNode.add(tree_window);
		Editor.addElement(id, new_element);

		if (new_element.getComponent() != null && parent != null && !parent.equals(new_element)) {
			((WindowVisualiser)root.getComponent()).add(new_element.getComponent());
		}

		Element elements = elem.getChild("elements");
		if (elements != null) {
			Iterator iterator = elements.getChildren().iterator();
			while (iterator.hasNext()) {
				Element child = (Element) iterator.next();
				handleNode(child, tree_window, (XElementBranch)new_element);
			}
		}
	}

	private void loadFile(String location) {
		setCursor(Cursor.WAIT_CURSOR);
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(new FileInputStream(location));

			Element xgui = doc.getRootElement();

			// Load the 'gui' element
			Element window = xgui.getChild("elements").getChild("XWindow");

			handleNode(window, tree_top, root);

			desktop.removeAll();
			desktop.add(root.getComponent());
			root.getComponent().show();
			((XWindow)root).transformResize();

			DefaultMutableTreeNode currentNode = tree_top.getNextNode();
			do {
				if (currentNode.getLevel() >= 1)
					tree.expandPath(new TreePath(currentNode.getPath()));
				currentNode = currentNode.getNextNode();
			} while (currentNode != null);


			// Load the 'slots' element
			Iterator iterator = xgui.getChild("slots").getChildren().iterator();
			while (iterator.hasNext()) {
				Element slot = (Element)iterator.next();
				XSlot xslot = new XSlot(slot.getAttribute("name").getValue());
				Iterator paramIterator = slot.getChild("parameters").getChildren("parameter").iterator();
				while (paramIterator.hasNext()) {
					Element param = (Element)paramIterator.next();
					xslot.addParam(param.getAttribute("name").getValue(), XSlot.ParamType.valueOf(param.getAttribute("type").getValue()));
				}
				//slots.add(xslot);
				Editor.addSlot(xslot);
			}


			//Load the 'slot_mappings' element
			Iterator slotMappingsIterator = xgui.getChild("slot_mappings").getChildren().iterator();
			while (slotMappingsIterator.hasNext()) {
				Element slot_mapping = (Element)slotMappingsIterator.next();
				String mapping_name = slot_mapping.getAttribute("name").getValue();
				String slot_name = slot_mapping.getChild("slot").getValue();
				XSlotMapping mapping = new XSlotMapping(mapping_name, slot_name);
				Iterator paramIterator = slot_mapping.getChild("bindings").getChildren("binding").iterator();
				while (paramIterator.hasNext()) {
					Element binding = (Element)paramIterator.next();
					String param_name = binding.getChild("parameter").getValue();
					String element_name = binding.getChild("element").getValue();
					String property_name = binding.getChild("property").getValue();
					mapping.addParamMapping(param_name, element_name, property_name);
				}
				Editor.addSlotMapping(mapping_name, mapping);
			}

			//Load the 'connections' element
			Iterator connsIterator = xgui.getChild("connections").getChildren().iterator();
			while (connsIterator.hasNext()) {
				Element slot_mapping = (Element)connsIterator.next();
				Editor.getElements().get(slot_mapping.getChild("element").getValue()).getSignal(slot_mapping.getChild("signal").getValue()).setMappingName(slot_mapping.getChild("slot_mapping").getValue());
			}

		} catch(Exception ex) {
			System.out.println("ERROR: " + ex.getMessage());
			System.out.println("ERROR: " + ex.getLocalizedMessage());
			ex.printStackTrace();
		}
		setCursor(Cursor.DEFAULT_CURSOR);
	}

	private void generateXml(XElement element, Element parent) {
		Element current_element = new Element(element.getClass().getSimpleName());
		current_element.setAttribute(new Attribute("id", element.getProperty("id").getValue().toString()));
		for (XProperty prop : element.getProperties()) {
			if (!prop.getName().equals("id"))
				current_element.addContent(new Element(prop.getName()).setText(prop.getDisplayValue()));
		}
		parent.addContent(current_element);

		if (element.hasChildren()) {
			Element elements = new Element("elements");
			current_element.addContent(elements);
			XElement[] children = ((XElementBranch)element).getChildren();
			for (int i = 0; i < children.length; i++) {
				generateXml(children[i], elements);
			}
		}
	}

	private void saveFile(String location) {
		Element xgui_root = new Element("xgui");

		// Generate XML for gui elements
		Element elements = new Element("elements");
		generateXml(root, elements);
		xgui_root.addContent(elements);

		// Generate XML for slots
		Element slotsElement = new Element("slots");
		xgui_root.addContent(slotsElement);
		for (XSlot slot : Editor.getSlots()) {
			slotsElement.addContent(slot.toXML());
		}

		// Generate XML for slot mappings
		Element slotMappingsElement = new Element("slot_mappings");
		xgui_root.addContent(slotMappingsElement);
		for (Map.Entry<String, XSlotMapping> entry : Editor.getSlotMappings().entrySet()) {
			slotMappingsElement.addContent(entry.getValue().toXML());
		}

		// Generate XML for connections
		Element connectionsElement = new Element("connections");
		xgui_root.addContent(connectionsElement);

		for (Map.Entry<String, XElement> entry : Editor.getElements().entrySet()) {
			String element_name = entry.getKey();
			for (XSignal signal : entry.getValue().getSignals()) {
				if (signal.hasMapping()) {
					Element connection = new Element("connection");
					connection.addContent(new Element("element").setText(element_name));
					connection.addContent(new Element("signal").setText(signal.getName()));
					connection.addContent(new Element("slot_mapping").setText(signal.getMappingName()));
					connectionsElement.addContent(connection);
				}
			}
		}


		// Generate XML document and write to file
		Document doc = new Document(xgui_root);
		XMLOutputter xmlOutput = new XMLOutputter();
		xmlOutput.setFormat(Format.getPrettyFormat());
		try {
			xmlOutput.output(doc, new FileWriter(location));
		} catch (Exception ex) {
			System.out.println("SAVE FAILED");
		}
	}

	private void reset() {
		tree_top.removeAllChildren();
		root = new XWindow("Window1", "Window1", 400, 300, true, true, true, true, true);
		DefaultMutableTreeNode tree_window = new DefaultMutableTreeNode(root);
		tree_top.add(tree_window);
		tree.expandRow(0);
		tree.clearSelection();
		tree.updateUI();

		IDHandler.reset();
		IDHandler.add("Window1");
		propertyModel.clear();
		signalModel.clear();
		selectedElement = null;
		save_location = null;
		Editor.clearSlots();
		Editor.clearElements();
		Editor.clearSlotMappings();

		desktop.removeAll();
		desktop.add(root.getComponent());
		root.getComponent().show();
		((XWindow)root).transformResize();

		this.setTitle("xGUI Editor");
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == menu_file_new) {
			reset();
		} else if (e.getSource() == menu_file_open) {
			JFileChooser fileChooser = new JFileChooser("C:\\Users\\James\\Documents\\Uni Work\\Year 3\\CM0343 - Individual Project\\Code");
			fileChooser.setDialogTitle("Choose Project");
			if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
				reset();
				root = null;
				tree_top.removeAllChildren();
				save_location = fileChooser.getSelectedFile().getAbsolutePath();
				loadFile(save_location);
				this.setTitle(fileChooser.getSelectedFile().getName() + " - xGUI Editor");
			}
		} else if (e.getSource() == menu_file_save) {
			if (save_location == null) {
				JFileChooser fileChooser = new JFileChooser("C:\\Users\\James\\Documents\\Uni Work\\Year 3\\CM0343 - Individual Project\\Code");
				fileChooser.setDialogTitle("Choose Project Save Location");
				if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
					save_location = fileChooser.getSelectedFile().getAbsolutePath();
					saveFile(save_location);
					this.setTitle(fileChooser.getSelectedFile().getName() + " - xGUI Editor");
				}
			} else {
				saveFile(save_location);
			}
		} else if (e.getSource() == menu_file_saveas) {
			JFileChooser fileChooser = new JFileChooser("C:\\Users\\James\\Documents\\Uni Work\\Year 3\\CM0343 - Individual Project\\Code");
			fileChooser.setDialogTitle("Choose Project Save Location");
			if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
				save_location = fileChooser.getSelectedFile().getAbsolutePath();
				saveFile(save_location);
				this.setTitle(fileChooser.getSelectedFile().getName() + " - xGUI Editor");
			}
		} else if (e.getSource() == menu_file_exit) {
			dispose();
		} else if (e.getSource() == menu_edit_slots) {
			new SlotsDialog().setVisible(true);
		} else if (e.getSource() == menu_elements_remove) {
			removeSelectedElement();
		}
	}
	
	private void removeSelectedElement() {
		if (selectedElement != null) {
			int n = JOptionPane.showConfirmDialog(this, "Are you sure you want to remove '"+selectedElement.getProperty("id").getValue().toString()+"'?", "Remove Element", JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				Object[] path = tree.getSelectionPath().getPath();
				((DefaultMutableTreeNode)path[path.length-1]).removeFromParent();
				tree.updateUI();
				((XElementBranch)((DefaultMutableTreeNode)path[path.length-2]).getUserObject()).removeChild(selectedElement.getProperty("id").getValue().toString());
				IDHandler.remove(selectedElement.getProperty("id").getValue().toString());
				((WindowVisualiser)root.getComponent()).remove(selectedElement.getComponent());
				selectedElement = null;
			}
		}
	}

	private void arrangeSplitPanels() {
		splitPane2.setDividerLocation(0.6);
		splitPaneRight.setDividerLocation(0.95);
		splitPane.setDividerLocation(0.25);
		splitPane2.setResizeWeight(0.6);
		splitPaneRight.setResizeWeight(0.5);
		splitPane.setResizeWeight(0.25);
		try {
			((WindowVisualiser)root.getComponent()).setSelected(true);
		} catch(Exception ex) {
			System.out.println("ERROR: arrangeSplitPanels");
		}
	}

}