import java.util.Map;
import java.util.LinkedHashMap;
import org.jdom2.*;
import org.jdom2.input.*;
import org.jdom2.output.*;

class ElementFactory {
	private static Map<String, Class> elementMap;
	private static Map<String, Class> propertyMap;

	static {
		elementMap = new LinkedHashMap<String, Class>();
		elementMap.put("XButton", XButton.class);
		elementMap.put("XWindow", XWindow.class);
		elementMap.put("XGroupBox", XGroupBox.class);
		elementMap.put("XTextBox", XTextBox.class);
		elementMap.put("XLabel", XLabel.class);
		elementMap.put("XImageBox", XImageBox.class);

		propertyMap = new LinkedHashMap<String, Class>();
		propertyMap.put("id", XPropertyID.class);
		propertyMap.put("text", XPropertyString.class);
		propertyMap.put("width", XPropertyInteger.class);
		propertyMap.put("height", XPropertyInteger.class);
		propertyMap.put("top", XPropertyInteger.class);
		propertyMap.put("left", XPropertyInteger.class);
		propertyMap.put("resizable", XPropertyBoolean.class);
		propertyMap.put("closable", XPropertyBoolean.class);
		propertyMap.put("minimizable", XPropertyBoolean.class);
		propertyMap.put("maximizable", XPropertyBoolean.class);
		propertyMap.put("imagesource", XPropertyBitmap.class);
		propertyMap.put("halign", XPropertyHorizontalAlign.class);
		propertyMap.put("valign", XPropertyVerticalAlign.class);
	}

	public static XElement createDefault(String type) {
		XElement new_element = null;
		if (type.equals("Button")) {
			String id = IDHandler.getNextID("Button");
			new_element = new XButton(id, id, 100, 30, 0, 0);
		} else if (type.equals("Text Box")) {
			String id = IDHandler.getNextID("TextBox");
			new_element = new XTextBox(id, id, 150, 24, 0, 0);
		} else if (type.equals("Group Box")) {
			String id = IDHandler.getNextID("GroupBox");
			new_element = new XGroupBox(id, id, 250, 200, 0, 0);
		} else if (type.equals("Image Box")) {
			String id = IDHandler.getNextID("ImageBox");
			new_element = new XImageBox(id, 100, 100, 0, 0);
		} else if (type.equals("Label")) {
			String id = IDHandler.getNextID("Label");
			new_element = new XLabel(id, id, 150, 20, 0, 0);
		} else if (type.equals("Panel")) {
			String id = IDHandler.getNextID("Panel");
			new_element = new XPanel(id, 250, 250, 0, 0);
		}
		return new_element;
	}

	public static XElement create(Element element) {
		try {
			XElement new_element = (XElement)elementMap.get(element.getName()).newInstance();
			new_element.setProperty("id", new XPropertyID("id", "ID", element.getAttribute("id").getValue()));
			for (Element elem : element.getChildren()) {
				String name = elem.getName();
				if (!name.equals("elements")) {
					XProperty property = (XProperty)propertyMap.get(name).getDeclaredConstructor(String.class, String.class, String.class).newInstance(name, name, elem.getValue());
					new_element.setProperty(name, property);
				}
			}
			new_element.generateComponent();
			return new_element;
		} catch (Exception ex) {
			System.out.println("Factory Exception: " + ex);
			return null;
		}
	}
}