import javax.swing.table.AbstractTableModel;
import javax.swing.JOptionPane;
import javax.swing.DefaultCellEditor;

class PropertyTableModel extends AbstractTableModel {
	private String[] columnNames = { "Property", "Value" };
	private XProperty[] properties = new XProperty[0];

	public void clear() {
		this.properties = new XProperty[0];
		fireTableStructureChanged();
	}

	public void update(XProperty[] properties) {
		System.out.println("update fired");
		this.properties = properties;
		fireTableStructureChanged();
	}

	public String getPropertyName(int index) {
		return this.properties[index].getName();
	}

	public Class getClassAt(int index) {
		return this.properties[index].getClass();
	}

	public DefaultCellEditor getCellEditor(int index) {
		return this.properties[index].getCellEditor();
	}

	public Object getPropertyValue(int index) {
		return this.properties[index].getValue();
	}

	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return this.properties.length;
	}

	public String getColumnName(int col) {
		return columnNames[col];
	}

	public Object getValueAt(int row, int col) {
		return (col == 0 ? properties[row].getFriendlyName() : properties[row].getDisplayValue());
	}

	public boolean isCellEditable(int row, int col) {
		return (col == 1);
	}

	public void setValueAt(Object value, int row, int col) {
		System.out.println("setvalueAt fired");
		try {
			String val = value.toString();
			if (this.properties[row].getName().equals("id")) {
				if (!val.equals(this.properties[row].getValue())) {
					if (!IDHandler.isUsed(val)) {
						IDHandler.add(val);
						IDHandler.remove(this.properties[row].getValue().toString());
						Editor.pushTreeUpdate();
						this.properties[row].setValueFromString(val);
						fireTableCellUpdated(row, col);
						Editor.pushSelectedComponentUpdate();
					} else {
						JOptionPane.showMessageDialog(Editor.getEditorWindowFrame(), "This ID is already in use by another element.", "Invalid Property", JOptionPane.WARNING_MESSAGE);
					}
				}
			} else {
				this.properties[row].setValueFromString(val);
				fireTableCellUpdated(row, col);
				Editor.pushSelectedComponentUpdate();
			}
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(Editor.getEditorWindowFrame(), "The value you provided is invalid for this property.", "Invalid Property", JOptionPane.WARNING_MESSAGE);
		}
	}
}