import javax.swing.*;
import java.awt.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Arrays;
import java.util.ArrayList;
import javax.swing.tree.DefaultMutableTreeNode;
import java.util.Map.Entry;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;

public class SlotsDialog extends JDialog {
	JTree slotsTree;
	JButton addSlot, addParam, delete, close;
	DefaultMutableTreeNode root;
	
	class SlotsTreeIconRenderer extends DefaultTreeCellRenderer {
		ImageIcon slot, param;
		
		public SlotsTreeIconRenderer() {
			slot = new ImageIcon(SlotsTreeIconRenderer.class.getResource("/icons/slot.png"));
			param = new ImageIcon(SlotsTreeIconRenderer.class.getResource("/icons/param.png"));
		}

		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
			super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
			setIcon(((DefaultMutableTreeNode)value).getLevel() == 1 ? slot : param);
			return this;
		}
	}
	
	public SlotsDialog() {
		super(Editor.getEditorWindowFrame(), Dialog.ModalityType.APPLICATION_MODAL);
		setTitle("Slots");
		setSize(new Dimension(500, 400));
		setResizable(false);
		setLayout(new BorderLayout());
		setLocationRelativeTo(Editor.getEditorWindowFrame());

		//this.owner = Editor.editorWindow;

		root = new DefaultMutableTreeNode("Root");
		
		for (XSlot slot : Editor.getSlots()) {
			DefaultMutableTreeNode slotNode = new DefaultMutableTreeNode(slot.getName());
			for (Entry<String, XSlot.ParamType> param : slot.getParams().entrySet()) {
				DefaultMutableTreeNode paramNode = new DefaultMutableTreeNode(param.getKey() + " : " + param.getValue());
				slotNode.add(paramNode);
			}
			root.add(slotNode);
		}



		slotsTree = new JTree(root);
		slotsTree.setRootVisible(false);
		slotsTree.setRowHeight(19);
		slotsTree.setFont(Editor.getUIFont());
		slotsTree.setBorder(new javax.swing.border.EmptyBorder(3, 3, 3, 3));
		slotsTree.setCellRenderer(new SlotsTreeIconRenderer());
		
		slotsTree.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)slotsTree.getLastSelectedPathComponent();
				System.out.println(node.getLevel());
				if (node.getLevel() == 1) {
					//addParam.setEnabled(false);
					delete.setLabel("Delete Slot");
				} else {
					//addParam.setEnabled(true);
					delete.setLabel("Delete Parameter");
				}
			}
		});
		
		JScrollPane treePanel = new JScrollPane(slotsTree);
		treePanel.setBorder(BorderFactory.createEmptyBorder());

		add(treePanel, BorderLayout.CENTER);
		
		JPanel controlsPanel = new JPanel();
		FlowLayout controlLayout = new FlowLayout(FlowLayout.LEFT);
		controlsPanel.setLayout(controlLayout);
		
		addSlot = new JButton("Add Slot...");
		addSlot.setFont(Editor.getUIFont());
		addParam = new JButton("Add Parameter...");
		addParam.setFont(Editor.getUIFont());
		//addParam.setEnabled(false);
		delete = new JButton("Delete Slot");
		delete.setFont(Editor.getUIFont());
		//delete.setEnabled(false);
		close = new JButton("Close");
		close.setFont(Editor.getUIFont());

		final Window _this = this;
		
		ActionListener controlActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (e.getSource() == addSlot) {
					AddSlotDialog asd = new AddSlotDialog(_this, AddSlotDialog.AddSlotDialogMode.Add);
					String ret = asd.showDialog();
					if (ret != null) {
						Editor.addSlot(new XSlot(ret));
						updateTree();
					}
				} else if (e.getSource() == addParam) {
					AddParamDialog asd = new AddParamDialog(_this, AddParamDialog.AddParamDialogMode.Add);
					Object[] ret = asd.showDialog();
					if (ret != null) {
						//Editor.editorWindow.slots.get(getSlotIndex()).addParam(ret[0].toString(), (XSlot.ParamType)ret[1]);
						Editor.getSlots().get(getSlotIndex()).addParam(ret[0].toString(), (XSlot.ParamType)ret[1]);
						System.out.println(Arrays.toString(ret));
						updateTree();
					}
				} else if (e.getSource() == delete) {
					int slotIndex = getSlotIndex();
					
					
					System.out.println("You clicked the delete button!");
				} else if (e.getSource() == close) {
					dispose();
				}
			}
		};

		addSlot.addActionListener(controlActionListener);
		addParam.addActionListener(controlActionListener);
		delete.addActionListener(controlActionListener);
		close.addActionListener(controlActionListener);

		controlsPanel.add(addSlot);
		controlsPanel.add(addParam);
		controlsPanel.add(delete);
		controlsPanel.add(close);
		add(controlsPanel, BorderLayout.PAGE_END);
	}

	private int getSlotIndex() {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)slotsTree.getLastSelectedPathComponent();
		if (node.getLevel() == 2)
			node = (DefaultMutableTreeNode)node.getParent();
		return node.getParent().getIndex(node);
	}
	
	private void updateTree() {
		root.removeAllChildren();
		for (XSlot slot : Editor.getSlots()) {
			DefaultMutableTreeNode slotNode = new DefaultMutableTreeNode(slot.getName());
			for (Entry<String, XSlot.ParamType> param : slot.getParams().entrySet()) {
				DefaultMutableTreeNode paramNode = new DefaultMutableTreeNode(param.getKey() + " : " + param.getValue());
				slotNode.add(paramNode);
			}
			root.add(slotNode);
		}
		slotsTree.updateUI();
	}
	
	//public static ArrayList<XSlot> slots = new ArrayList<XSlot>();
}
