import javax.swing.JButton;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.Point;
import javax.swing.SwingUtilities;
import java.awt.Cursor;

public class XButton extends XElementLeaf {
	public XButton() {
		this.setSignal("Click", new XSignal("Click"));
	}

	public XButton(String id, String text, int width, int height, int top, int left) {
		super(id);

		this.setProperty("left", new XPropertyInteger("left", "Left", left));
		this.setProperty("top", new XPropertyInteger("top", "Top", top));
		this.setProperty("width", new XPropertyInteger("width", "Width", width));
		this.setProperty("height", new XPropertyInteger("height", "Height", height));
		this.setProperty("text", new XPropertyString("text", "Text", text));

		this.setSignal("click", new XSignal("click"));
		this.setSignal("created", new XSignal("created"));

		this.generateComponent();
	}

	public void generateComponent() {
		final JButton button = new JButton();
		button.setRolloverEnabled(false);
		button.setFocusable(false);
		button.setCursor(new Cursor(Cursor.MOVE_CURSOR));
		this.component = (java.awt.Component)button;
		this.updateComponent();

		final Point offset = new Point(-1, -1);
		
		button.addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent evt) {
				if (offset.getX() == -1 && offset.getY() == -1) {
					offset.setLocation(MouseInfo.getPointerInfo().getLocation());
					SwingUtilities.convertPointFromScreen(offset, button);
				}
				move(MouseInfo.getPointerInfo().getLocation(), offset);
			}
		});

		button.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				offset.setLocation(-1, -1);
			}
		});
	}

	public void updateComponent() {
		JButton button = ((JButton)this.component);
		button.setLabel((String)this.getProperty("text").getValue());
		button.setLocation(
			(Integer)this.getProperty("left").getValue(),
			(Integer)this.getProperty("top").getValue()
		);
		button.setSize(new Dimension(
			(Integer)this.getProperty("width").getValue(),
			(Integer)this.getProperty("height").getValue()
		));
	}

	private void move(Point position, Point offset) {
		JButton button = ((JButton)this.component);
		SwingUtilities.convertPointFromScreen(position, Editor.getEditorWindow().getWindowVisualiserContentPane());
		position.translate((int)offset.getX() * -1, (int)offset.getY() * -1);
		position.setLocation((int)(Math.rint((double) position.getX() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE),
								(int)(Math.rint((double) position.getY() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE));
		button.setLocation(position);
		this.getProperty("left").setValue((int)(position.getX()));
		this.getProperty("top").setValue((int)(position.getY()));
	}
}