import javax.swing.JPanel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.Cursor;
import javax.swing.SwingUtilities;
import javax.swing.BorderFactory;
import java.awt.Color;

public class XGroupBox extends XElementBranch {
	public XGroupBox() { }

	public XGroupBox(String id, String text, int width, int height, int top, int left) {
		super(id);
		this.setProperty("width", new XPropertyInteger("width", "Width", width));
		this.setProperty("height", new XPropertyInteger("height", "Height", height));
		this.setProperty("top", new XPropertyInteger("top", "Top", top));
		this.setProperty("left", new XPropertyInteger("left", "Left", left));
		this.setProperty("text", new XPropertyString("text", "Text", text));
		this.generateComponent();
	}

	public void generateComponent() {
		final JPanel panel = new JPanel();
		panel.setFocusable(false);
		panel.setCursor(new Cursor(Cursor.MOVE_CURSOR));
		this.component = (java.awt.Component)panel;
		this.updateComponent();

		final Point offset = new Point(-1, -1);
		
		panel.addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent evt) {
				if (offset.getX() == -1 && offset.getY() == -1) {
					offset.setLocation(MouseInfo.getPointerInfo().getLocation());
					SwingUtilities.convertPointFromScreen(offset, panel);
				}
				move(MouseInfo.getPointerInfo().getLocation(), offset);
			}
		});

		panel.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				offset.setLocation(-1, -1);
			}
		});
	}

	public void updateComponent() {
		JPanel panel = ((JPanel)this.component);
		panel.setBorder(BorderFactory.createTitledBorder(this.getProperty("text").getValue().toString()));
		panel.setLocation(
			(Integer)this.getProperty("left").getValue(),
			(Integer)this.getProperty("top").getValue()
		);
		panel.setSize(new Dimension(
			(Integer)this.getProperty("width").getValue(),
			(Integer)this.getProperty("height").getValue()
		));
	}

	private void move(Point position, Point offset) {
		JPanel panel = ((JPanel)this.component);
		SwingUtilities.convertPointFromScreen(position, Editor.getEditorWindow().getWindowVisualiserContentPane());
		position.translate((int)offset.getX() * -1, (int)offset.getY() * -1);
		position.setLocation((int)(Math.rint((double) position.getX() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE),
								(int)(Math.rint((double) position.getY() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE));
		panel.setLocation(position);
		this.getProperty("left").setValue((int)(position.getX()));
		this.getProperty("top").setValue((int)(position.getY()));
	}
}