import javax.swing.JLabel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.Point;
import javax.swing.SwingUtilities;
import java.awt.Cursor;
import java.awt.dnd.DropTarget;

public class XLabel extends XElementLeaf {
	public XLabel() {
		super("id");
	}

	public void generateComponent() {
		final JLabel label = new JLabel();
		label.setFocusable(false);
		label.setCursor(new Cursor(Cursor.MOVE_CURSOR));
		DropTarget d = new DropTarget();
		d.setActive(false);
		label.setDropTarget(d);
		this.component = (java.awt.Component)label;
		this.updateComponent();

		final Point offset = new Point(-1, -1);

		label.addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent evt) {
				if (offset.getX() == -1 && offset.getY() == -1) {
					offset.setLocation(MouseInfo.getPointerInfo().getLocation());
					SwingUtilities.convertPointFromScreen(offset, label);
				}
				move(MouseInfo.getPointerInfo().getLocation(), offset);
			}
		});

		label.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				offset.setLocation(-1, -1);
			}
		});
	}

	public XLabel(String id, String text, int width, int height, int top, int left) {
		super(id);

		this.setProperty("left", new XPropertyInteger("left", "Left", left));
		this.setProperty("top", new XPropertyInteger("top", "Top", top));
		this.setProperty("width", new XPropertyInteger("width", "Width", width));
		this.setProperty("height", new XPropertyInteger("height", "Height", height));
		this.setProperty("text", new XPropertyString("text", "Text", text));
		
		final JLabel label = new JLabel(text);
		label.setSize(new Dimension(width, height));
		label.setLocation(left, top);
		label.setFocusable(false);
		label.setCursor(new Cursor(Cursor.MOVE_CURSOR));
		DropTarget d = new DropTarget();
		d.setActive(false);
		label.setDropTarget(d);
		this.component = (java.awt.Component)label;

		final Point offset = new Point(-1, -1);

		label.addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent evt) {
				if (offset.getX() == -1 && offset.getY() == -1) {
					offset.setLocation(MouseInfo.getPointerInfo().getLocation());
					SwingUtilities.convertPointFromScreen(offset, label);
				}
				move(MouseInfo.getPointerInfo().getLocation(), offset);
			}
		});

		label.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				offset.setLocation(-1, -1);
			}
		});
	}
	
	private void move(Point position, Point offset) {
		JLabel label = ((JLabel)this.component);
		SwingUtilities.convertPointFromScreen(position, Editor.getEditorWindow().getWindowVisualiserContentPane());
		position.translate((int)offset.getX() * -1, (int)offset.getY() * -1);
		position.setLocation((int)(Math.rint((double) position.getX() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE),
								(int)(Math.rint((double) position.getY() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE));
		label.setLocation(position);
		this.getProperty("left").setValue((int)(position.getX()));
		this.getProperty("top").setValue((int)(position.getY()));
	}
	
	public void updateComponent() {
		JLabel label = ((JLabel)this.component);
		label.setText((String)this.getProperty("text").getValue());
		label.setLocation(
			(Integer)this.getProperty("left").getValue(),
			(Integer)this.getProperty("top").getValue()
		);
		label.setSize(new Dimension(
			(Integer)this.getProperty("width").getValue(),
			(Integer)this.getProperty("height").getValue()
		));
	}
}