import javax.swing.JFrame;
import javax.swing.JDialog;
import java.awt.Dimension;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Arrays;
import org.jdom2.*;
import org.jdom2.input.*;
import org.jdom2.output.*;

public class XSlot {
	public enum ParamType {
		String, Integer, Double, Float, Boolean
	}

	private String name;
	private Map<String, ParamType> params;

	public XSlot(String name) {
		this.name = name;
		this.params = new LinkedHashMap<String, ParamType>();
	}

	public void addParam(String name, ParamType paramType) {
		this.params.put(name, paramType);
	}

	public void removeParam(String name) {
		this.params.remove(name);
	}

	public int paramCount() {
		return this.params.size();
	}

	public String toString() {
		return "XSlot:" + this.name + "(" + Arrays.toString(this.params.values().toArray()) + ")";
	}

	public String getName() {
		return this.name;
	}

	public Map<String, ParamType> getParams() {
		return this.params;
	}

	public Element toXML() {
		Element slotElement = new Element("slot");
		slotElement.setAttribute(new Attribute("name", this.name));

		Element parameters = new Element("parameters");
		slotElement.addContent(parameters);

		for (Map.Entry<String, XSlot.ParamType> entry : this.params.entrySet()) {
			Element parameter = new Element("parameter");
			parameter.setAttribute(new Attribute("name", entry.getKey()));
			parameter.setAttribute(new Attribute("type", entry.getValue().toString()));
			parameters.addContent(parameter);
		}
		return slotElement;
	}
}
